unit PuzzleUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtDlgs, Menus, ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    MainMenu1: TMainMenu;
    FileMnu: TMenuItem;
    FileOpenMnu: TMenuItem;
    FileExitMnu: TMenuItem;
    OpenPictureDialog1: TOpenPictureDialog;
    ImageMnu: TMenuItem;
    ImageRemixMnu: TMenuItem;
    ImageViewMnu: TMenuItem;
    AboutMnu: TMenuItem;
    ImageRowColNumsMnu: TMenuItem;
    Label1: TLabel;
    procedure FileOpenMnuClick(Sender: TObject);
    procedure ImageRemixMnuClick(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FileExitMnuClick(Sender: TObject);
    procedure ImageViewMnuClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpAboutMnuClick(Sender: TObject);
    procedure ImageRowColNumsMnuClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure AboutMnuClick(Sender: TObject);
  private
    { Private declarations }
    BitVal : array[0..9, 0..9] of byte;
    Piece : TBitMap;
    frHeight, frWidth : word;
    procedure ZeroBitVal;
    function PutBack : boolean;
    procedure PushImage (x1, y1 : byte; x2, y2 : SmallInt);
    procedure Won;
  public
    { Public declarations }
    OrigBitMap : TBitMap;
    xCord, yCord : word;
    Rows, Cols : Byte;
    procedure MixUP;
    procedure Changefr;
  end;

procedure SwapByte (var num1, num2 : byte);

var
  Form1: TForm1;

implementation

uses ViewUnit, AboutUnit, RowColUnit;

{$R *.DFM}

procedure SwapByte (var num1, num2 : byte);
var hold : Byte;
begin
  hold := num1;
  num1 := num2;
  num2 := hold;
end;

function TForm1.PutBack : boolean;
var
  i, x, y : byte;
begin
  i := 0;

  for x := 0 to cols - 1 do
    for y := 0 to rows - 1 do begin
      inc (i);
      if BitVal[x, y] <> i then begin
        result := false;
        exit;
      end;
    end;

  result := true;
end;

procedure TForm1.ZeroBitVal;
var
  x, y : byte;
begin
  for x := 0 to cols - 1 do
    for y := 0 to rows - 1 do
      BitVal[x, y] := 0;
end;

procedure TForm1.MixUp;
var
  BitMap : TBitMap;
  i, x, y, x1, y1 : byte;
begin
  BitMap := TBitMap.Create;
  BitMap.Width := OrigBitMap.Width;
  BitMap.Height := OrigBitMap.Height;

  ZeroBitVal;

  i := 0;
  for x := 0 to cols - 1 do
    for y := 0 to rows - 1 do begin
      repeat
        x1 := random (cols);
        y1 := random (rows);
      until BitVal[x1, y1] = 0;
      inc (i);
      BitVal[x1, y1] := i;
      if (x = cols - 1) and (y = rows - 1) then begin
        Piece.Height := frHeight;
        Piece.Width := frWidth;
        Piece.Canvas.CopyRect (Rect (0, 0, frWidth, frHeight),
                               OrigBitMap.Canvas,
                               Rect (x * frWidth, y * frHeight, (x + 1) * frWidth, (y + 1) * frHeight));
      end else
        BitMap.Canvas.CopyRect (Rect (x1 * frWidth, y1 * frHeight, (x1 + 1) * frWidth, (y1 + 1) * frHeight),
                                OrigBitMap.Canvas,
                                Rect (x * frWidth, y * frHeight, (x + 1) * frWidth, (y + 1) * frHeight));
    end;

  Image1.Picture.BitMap := BitMap;
  BitMap.Free;
end;

procedure TForm1.Changefr;
begin
  frHeight := OrigBitMap.Height div rows;
  frWidth := OrigBitMap.Width div cols;
end;

procedure TForm1.FileOpenMnuClick(Sender: TObject);
begin
  if not OpenPictureDialog1.Execute then exit;

  if ViewForm.Visible then ViewForm.untile;

  OrigBitMap.LoadFromFile (OpenPictureDialog1.FileName);
  ViewForm.Load;

  Changefr;

  Height :=  OrigBitMap.Height + 47;
  Width := OrigBitMap.Width + 9;
  Position := poScreenCenter;
  xCord := Left; yCord := Top;

  ViewForm.Width := Form1.Width;
  ViewForm.Height := Form1.Height;

  MixUp;

  if ViewForm.Visible then ViewForm.tile;
end;

procedure TForm1.ImageRemixMnuClick(Sender: TObject);
begin
  MixUp;
end;

procedure TForm1.PushImage (x1, y1 : byte; x2, y2 : SmallInt);
begin
  Image1.Canvas.CopyRect (Rect (x2 * frWidth, y2 * frHeight, (x2 + 1) * frWidth, (y2 + 1) * frHeight),
                          Image1.Canvas,
                          Rect (x1 * frWidth, y1 * frHeight, (x1 + 1) * frWidth, (y1 + 1) * frHeight));


  Image1.Canvas.FillRect (Rect (x1 * frWidth, y1 * frHeight, (x1 + 1) * frWidth, (y1 + 1) * frHeight));
  SwapByte (BitVal[x2, y2], BitVal[x1, y1]);
end;

procedure TForm1.Won;
begin
  BitVal[cols - 1][rows - 1] := 0;
  Image1.Canvas.Draw ((cols - 1) * frWidth, (rows - 1) * frHeight, Piece);
  ShowMessage ('You put it back together!');
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  x1, y1 : byte;
  x2, y2 : SmallInt;
begin
  if Button <> mbLeft then exit;

  if OpenPictureDialog1.FileName = '' then exit;
  if (Y < 0) or (Y > Image1.Height) or (X < 0) or (X > Image1.Width) then exit;

  x1 := X div frWidth;
  y1 := Y div frHeight;

  for x2 := x1 - 1 to x1 + 1 do begin
    if (x2 < 0) or (x2 >= cols) then continue;

    if BitVal[x2, y1] = rows * cols then begin
      PushImage (x1, y1, x2, y1);

      if PutBack then won;

      exit;
    end;
  end;

  for y2 := y1 - 1 to y1 + 1 do begin
    if (y2 < 0) or (y2 >= rows) then continue;

    if BitVal[x1, y2] = rows * cols then begin
      PushImage (x1, y1, x1, y2);

      if PutBack then won;

      exit;
    end;
  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  OrigBitMap.Free;
  Piece.Free;
end;

procedure TForm1.FileExitMnuClick(Sender: TObject);
begin
  OrigBitMap.Free;
  Piece.Free;
  Application.Terminate;
end;

procedure TForm1.ImageViewMnuClick(Sender: TObject);
begin
  if ImageViewMnu.Checked then begin
    ViewForm.Hide;
    ImageViewMnu.Checked := false;
  end else begin
    ViewForm.Show;
    ImageViewMnu.Checked := true;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  OrigBitMap := TBitMap.Create;
  Piece := TBitMap.Create;
  randomize;

  Rows := 4;
  Cols := 4;
end;

procedure TForm1.HelpAboutMnuClick(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TForm1.ImageRowColNumsMnuClick(Sender: TObject);
begin
  RowColForm.ShowModal;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  xCord := Left;
  yCord := Top;
end;

procedure TForm1.AboutMnuClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

end.

